//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <iostream>
#include <fcntl.h>
#include <unistd.h>
#include <expat.h>
#include <SDL_image.h>
#include <SDL.h>
#include <pgapplication.h>
#include "armysetlist.h"
#include "armyset.h"
#include "armyset_army.h"
#include "File.h"

#define BUFF_SIZE 8192

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

Armysetlist* Armysetlist::s_instance = 0;

Armysetlist* Armysetlist::getInstance()
{
    if (!s_instance)
        s_instance = new Armysetlist();

    return s_instance;
}

void Armysetlist::deleteInstance()
{
    if (s_instance)
        delete s_instance;

    s_instance = 0;
}

Armysetlist::Armysetlist()
	:map<string, Armyset*>()
{
}

Armysetlist::~Armysetlist()
{
}

Armyset* Armysetlist::getArmyset(string name)
{
	// this one is rather easy thanks to the STL
	// if the name entry doesn't exist, map creates one and initialises with 0

	return (*this)[name];
}

bool Armysetlist::loadArmy(string tag, XML_Helper* helper)
{
    string name;
    int production;
    int productionCost;
    int upkeep;
    int strength;
    int defense;
    int hitpoints;
    int moves;
    int vitality;
    unsigned int moveBonus;
    unsigned int armyBonus;

    helper->getData(name, "name");
    helper->getData(production, "production");
    helper->getData(productionCost, "production_cost");
    helper->getData(upkeep, "upkeep");
    helper->getData(strength, "strength");
    helper->getData(defense, "defense");
    helper->getData(hitpoints, "hitpoints");
    helper->getData(moves, "moves");
    helper->getData(vitality, "vitality");
    helper->getData(moveBonus, "move_bonus");
    helper->getData(armyBonus, "army_bonus");

    Armyset_army army;
    army.setName(name);
    army.setProduction(production);
    army.setProductionCost(productionCost);
    army.setUpkeep(upkeep);
    army.setMaxStrength(strength);
    army.setDefense(defense);
    army.setMaxHp(hitpoints);
    army.setMaxMoves(moves);
    army.setVitality(vitality);
    army.setMoveBonus(moveBonus);
    army.setArmyBonus(armyBonus);
    (*this)[d_loadName]->army.push_back(army);

	return true;
}

bool Armysetlist::loadArmyset(string name)
{
	debug("load_armyset()");

    d_loadName = name;

	//first, look if Armyset already exists
	if ((*this)[name] != 0)
    {
        return true;
    }

	//no, so create a new Armyset first
	(*this)[name] = new Armyset(name);

    XML_Helper helper(File::getArmyset(name), ios::in);
	helper.registerTag("army", SigC::slot((*this), &Armysetlist::loadArmy));
    helper.parse();

    // set the pixmaps of the army (split the armyGallery)
    SDL_Surface* armyGallery = File::getArmyPicture(d_loadName);
    SDL_PixelFormat* fmt = armyGallery->format;
    // don't use alpha information, just copy the channel! very important
    SDL_SetAlpha(armyGallery, 0, 0);
    for (int i = 0; i < 12; i++)
    {
        SDL_Surface* tmp = SDL_CreateRGBSurface(SDL_HWSURFACE,
                40, 40, fmt->BitsPerPixel, fmt->Rmask, fmt->Gmask,
                fmt->Bmask, fmt->Amask);

        SDL_BlitSurface(armyGallery, PG_Rect(40*i, 0, 40, 40).SDLRect(), tmp, 0);
        SDL_Surface* pixmap = SDL_DisplayFormatAlpha(tmp);
        SDL_FreeSurface(tmp);
        (*this)[d_loadName]->army[i].setPixmap(pixmap);
    }

    // not needed anymore
    SDL_FreeSurface(armyGallery);
	return true;
}

bool Armysetlist::eraseByName(string name)
{
	iterator it = find(name);
	//armyset exists
	if (it != end())
	{
	//	delete (*this)[name];
		erase(it);
		return true;
	}
	return false;
}

